package com.jaworski.security.handbook;

// Encapsulate the concept of a shuffled alphabet.
public class LetterSubstitution {
 public static final int NUM_LETTERS = 26;
 // Maps (0 - 26) to the indices of the substituted letters.
 // E.g., substitution[3] = 10 means that d (3) maps to k (10).
 protected int[] substitution = new int[NUM_LETTERS];

 public LetterSubstitution() {
  // Set the default (no change) substitution.
  for(int i=0;i<NUM_LETTERS;++i) {
   substitution[i] = i;
  }
 }
 
 // Retrieve the letter which is mapped to c.
 public char getLetter(char c) {
  c = Character.toUpperCase(c);
  if(isUpperCaseLetter(c)) {
   for(int i=0;i<NUM_LETTERS;++i) {
    if(substitution[i] == (c - 'A')) 
     return (char) (i+'A');
   }
  }
  return c;
 }
 
 // Find the letter that is substituted for c.
 public char getSubstitute(char c) {
  c = Character.toUpperCase(c);
  if(isUpperCaseLetter(c)) {
   c = (char) (substitution[c-'A'] + 'A');
  }
  return c;
 }
 
 // Is the character A-Z?
 public static boolean isUpperCaseLetter(char c) {
  if(c >= 'A' && c <= 'Z') return true;
  return false;  
 }
 
 // Substitute c2 for c1.
 public void mapTo(char c1, char c2) {
  c1=Character.toUpperCase(c1);
  c2=Character.toUpperCase(c2);
  if(isUpperCaseLetter(c1) && isUpperCaseLetter(c2)) {
   substitution[c1-'A'] = c2 - 'A';
  }
 }
 
 // Come up with a pseudo-random substitution.
 public void randomize() {
  boolean[] used = new boolean[NUM_LETTERS];
  for(int i=0;i<NUM_LETTERS;++i) {
   int n = ((int) (NUM_LETTERS*1000*Math.random())) % NUM_LETTERS;
   while (used[n]) {
    n = (n + 1) % NUM_LETTERS;
   };
   substitution[i] = n;
   used[i] = true;
  }
 }
 
 // Allow the substitution to be displayed.
 public String toString() {
  String s = "(";
  for(int i=0;i<NUM_LETTERS;++i) {
   s += substitution[i];
   if(i!=NUM_LETTERS-1) s+=",";
  }
  s += ")";
  return s;
 }
}