package com.jaworski.security.handbook;

import java.security.*;
import javax.crypto.*;

public class KeyGeneratorApp {
 // The algorithm's name
 private String algorithm = "";
 // The size of the key (in bits) to generate
 private int keySize = 0;
 // Placeholder for secret keys
 private SecretKey skey = null;
 // Placeholder for public key pairs
 private KeyPair keys = null;
 
 public static void main(String[] args) {
  if(args.length != 2) {
   String err = "Usage: com.jaworski.security.handbook";
   err += ".KeyGeneratorApp algorithmName keySize";
   System.out.println(err);
   System.exit(0);
  }
  // Convert the key size to an int value
  int size = (new Integer(args[1])).intValue();
  // Create a KeyGeneratorApp object
  KeyGeneratorApp app = new KeyGeneratorApp(args[0],size);
  // Generate the key or key pair
  app.generate();
  // Display the key or key pair
  app.display();
 }
 public KeyGeneratorApp(String algorithm, int keySize) {
  this.algorithm = algorithm;
  this.keySize = keySize;
 }
 private void generate() {
  try {
   // Try to generate a public key pair
   KeyPairGenerator kpg = KeyPairGenerator.getInstance(algorithm);
   kpg.initialize(keySize);
   keys = kpg.genKeyPair();
  }catch(NoSuchAlgorithmException ex1) {
   // Try to generate a secret key
   try {
    KeyGenerator kg = KeyGenerator.getInstance(algorithm);
    kg.init(keySize);
    skey = kg.generateKey();
   }catch(NoSuchAlgorithmException ex2) {
    System.out.println("Algorithm not supported: "+algorithm);
    System.exit(0);
   }
  }
 }
 private void display() {
  if(skey != null) {
   // Display the secret key
   System.out.println("Algorithm: "+algorithm);
   System.out.println("Key size: "+keySize);
   System.out.println("Secret key in "+skey.getFormat()+" format:");
   // Use Base64 encoding
   String skeyString = 
    Conversion.byteArrayToBase64String(skey.getEncoded());
   System.out.println(skeyString);
  }else if(keys != null) {
   // Display the public key pair
   System.out.println("Algorithm: "+algorithm);
   System.out.println("Key size: "+keySize);
   PrivateKey priKey = keys.getPrivate();
   PublicKey pubKey = keys.getPublic();
   System.out.println("Private key in "+priKey.getFormat()+" format:");
   // Use Base64 encoding
   String priKeyString = 
    Conversion.byteArrayToBase64String(priKey.getEncoded());
   System.out.println(priKeyString);
   System.out.println("Public key in "+pubKey.getFormat()+" format:");   
   String pubKeyString = 
    Conversion.byteArrayToBase64String(pubKey.getEncoded());
   System.out.println(pubKeyString);
  }
 }
}