package com.jaworski.security.handbook;

import java.security.*;

public class SignApp {
 static String errorMessage = 
  "Usage: com.jaworski.security.handbook.SignApp testString";
 public static void main(String[] args) {
  if(args.length < 1) 
   System.out.println(errorMessage);
  else{
   String alg = "DSA";
   KeyPair keyPair = generateKeyPair(alg);
   byte[] signature = performSigning(args[0],alg,keyPair);
   performVerification(args[0],alg,signature,keyPair.getPublic());
  }
 }
 static KeyPair generateKeyPair(String alg) {
  try {
   KeyPairGenerator kg = KeyPairGenerator.getInstance(alg);
   System.out.println("Generating key pair ...");
   KeyPair keyPair = kg.genKeyPair();
   return keyPair;
  }catch(Exception e) {
   System.out.println(e);
   System.exit(0);
  }
  return null;
 }
 static byte[] performSigning(String s,String alg,KeyPair keyPair) {
  try {
   Signature sign = Signature.getInstance(alg);
   PrivateKey privateKey = keyPair.getPrivate();
   PublicKey publicKey = keyPair.getPublic();
   sign.initSign(privateKey);
   System.out.println("Generating signature ...");
   sign.update(s.getBytes());
   byte[] b = sign.sign();
   System.out.println("\nMessage: "+s);
   System.out.println("Private key:\n"+
    Conversion.byteArrayToBase64String(privateKey.getEncoded()));
   System.out.println("Public key:\n"+
    Conversion.byteArrayToBase64String(publicKey.getEncoded()));
   System.out.println("Signature:\n"+
    Conversion.byteArrayToBase64String(b));
   return b;
  }catch(Exception e) {
   System.out.println(e);
   System.exit(0);
  }
  return null;
 }
 static void performVerification(String s, String alg,
   byte[] signature, PublicKey publicKey) {
  try {
   Signature sign = Signature.getInstance(alg);
   System.out.println("\nVerifying signature ...");
   sign.initVerify(publicKey);
   sign.update(s.getBytes());
   if(sign.verify(signature)) 
    System.out.println("Signature verified!");
   else
    System.out.println("Signature NOT verified!");
  }catch(Exception e){
   System.out.println(e);
   System.exit(0);
  }  
 }
}
