package com.jaworski.security.handbook;

import java.security.*;
import java.util.*;

public class SignedObjectApp {
 public static void main(String[] args) {
  String alg = "DSA";
  KeyPair keyPair = generateKeyPair(alg);
  Vector v = new Vector();
  v.add("This is a test!");
  SignedObject so = createSignedObject(v,alg,keyPair);
  performVerification(so,alg,keyPair.getPublic());
 }
 static KeyPair generateKeyPair(String alg) {
  try {
   KeyPairGenerator kg = KeyPairGenerator.getInstance(alg);
   System.out.println("Generating key pair ...");
   KeyPair keyPair = kg.genKeyPair();
   return keyPair;
  }catch(Exception e) {
   System.out.println(e);
   System.exit(0);
  }
  return null;
 }
 static SignedObject createSignedObject(Vector v,String alg,KeyPair keyPair) {
  try {
   Signature sign = Signature.getInstance(alg);
   System.out.println("Creating signed object ...");
   SignedObject so = 
    new SignedObject(v, keyPair.getPrivate(), sign);
   return so;
  }catch(Exception e) {
   System.out.println(e);
   System.exit(0);
  }
  return null;
 }
 static void performVerification(SignedObject so, String alg,
   PublicKey publicKey) {
  try {
   System.out.println("\nVerifying signature ...");
   Signature sign = Signature.getInstance(alg);
   if(so.verify(publicKey, sign)) 
    System.out.println("Signature verified!");
   else
    System.out.println("Signature NOT verified!");
  }catch(Exception e){
   System.out.println(e);
   System.exit(0);
  }  
 }
}
